#!/usr/bin/perl

# This program reads in the sessions for each user from the sequence data
# file and generates the similarity graph using LCS similarity
# and thresholding. The upper triangle of the similarity graph is written
# in a file to be converted to metis format and so on...

use Getopt::Std;

%option = ();
getopts("d:t:p:u:o:",\%option);
if( (!$option{d}) || (!$option{t}) || (!$option{o}) ){
  print "Usage:\n";
  print "LCS_sim.pl -d <data_filename> -t <threshold> -p <int power> -u <is_weighted> -o <output_filename>\n";
  exit(1);
}

%pageHash = ();
%timeHash = ();
%Time = ();
$sessionNum = 0;
$isWeighted = $option{u};

open(DATA,$option{d}) || die "couldn't open $option{d}:$!\n";

if($isWeighted == 1){
  while(<DATA>){
    chomp;
    $tmp = [ split ];  
    for($i=0;$i< ($#$tmp+1)/2; $i++){
      $pageHash{$sessionNum}[$i] = $$tmp[2*$i];
      $timeHash{$sessionNum}[$i] = $$tmp[2*$i+1];
      $Time{$sessionNum} += $$tmp[2*$i+1];
    }
    $sessionNum++;
  }
}
else{
  while(<DATA>){
    chomp;
    $tmp = [ split ];  
    for($i=0;$i< ($#$tmp+1); $i++){
      $pageHash{$sessionNum}[$i] = $$tmp[$i];
      $timeHash{$sessionNum}[$i] = 1;
      $Time{$sessionNum} += 1;
    }
    $sessionNum++;
  }
}
close(DATA);

$Num = $sessionNum;
print "Total : $Num sequences\n";
open(SIM,">$option{o}") || die "couldn't open $option{o}:$!\n";

for($i=0; $i<($Num-1); $i++){
  print "processing sequence $i ...\n";
  for($j=$i+1; $j<$Num; $j++){
    $total_sim = 0;
    @array1 = @{ $pageHash{$i} };
    @array2 = @{ $pageHash{$j} };
    @full_list = basicLCS( \@array1,\@array2 );	
    $length = ($#full_list+1)/3;
    unless($length == 0){
      @the_LCS = splice(@full_list,0,$length);
      @index_i = splice(@full_list,0,$length);
      @index_j = splice(@full_list,0,$length);
      $match = $#the_LCS;
      $time1 = 0;
      $time2 = 0;
      $sum_sim = 0;
      for ($index = 0;$index <= $match; $index++){
	$sum_sim += minmax($timeHash{$i}[$index_i[$index]],$timeHash{$j}[$index_j[$index]]);
	$time1 += $timeHash{$i}[$index_i[$index]];
	$time2 += $timeHash{$j}[$index_j[$index]];
      }

      # using AM rather than GM, since sq.root is a costly operation
      $sim = ($sum_sim/($match+1))*(((($time1/$Time{$i})+($time2/$Time{$j}))*0.5)**$option{p});
      if( $sim > $option{t} ){
	print SIM "$i $j $sim\n";
      }
    }
  } # for j loop ends 
}  # for i loop ends

close(SIM);


sub minmax{
  my @sorted = sort { $a <=> $b } @_;
  return ($sorted[0]/$sorted[1]);
}

sub basicLCS{
  my $s1_ref = $_[0];
  my $s2_ref = $_[1];
  
  my @the_list = ();
  my @c = ();
  my @b = ();
  
  my $length_s1=$#$s1_ref+1; 	
  my $length_s2=$#$s2_ref+1; 	


  for($ii=0; $ii <= $length_s1; $ii++ ){
    $c[$ii][0] = 0;
    $b[$ii][0] = 0;
  }
  for($jj=0; $jj <= $length_s2; $jj++ ){
    $c[0][$jj] = 0;
    $b[0][$jj] = 0;
  }
  
  
  for( $ii=1; $ii <= $length_s1; $ii++){
    for( $jj=1; $jj <= $length_s2; $jj++) {
      if($$s1_ref[$ii-1] == $$s2_ref[$jj-1]){
	$c[$ii][$jj] = $c[$ii-1][$jj-1] + 1;
	$b[$ii][$jj] = 0;
      }
      elsif($c[$ii-1][$jj] >= $c[$ii][$jj-1]){
	$c[$ii][$jj] = $c[$ii-1][$jj];
	$b[$ii][$jj] = 1;
      }
      else{
	$c[$ii][$jj] = $c[$ii][$jj-1];
	$b[$ii][$jj] = -1;
      }
    }	
  }	

  my @index1 = ();
  my @index2 = ();
  my @lcs = ();
  getIndex(\@b,$s1_ref,$length_s1,$length_s2,\@index1,\@index2,\@lcs);
  
  @the_list = (@lcs,@index1,@index2);
  
  return @the_list;
}


sub getIndex{
  my $b_ref = $_[0];
  my $s_ref = $_[1];
  my $I = $_[2];
  my $J = $_[3];
  $index1_ref = $_[4];
  $index2_ref = $_[5];
  $lcs_ref = $_[6];

  
  if( ($I == 0) || ($J == 0) ){
    return;
  }
  elsif( $$b_ref[$I][$J] == 0 ){
    getIndex($b_ref,$s_ref,$I-1,$J-1,$index1_ref,$index2_ref,$lcs_ref);
    push @$lcs_ref,$$s_ref[$I-1];
    push @$index1_ref,($I-1);   # the same old 0-1 starting problem, actually
    push @$index2_ref,($J-1);   # it should push_back I and J if it starts from 1
  }
  elsif($$b_ref[$I][$J] == 1 ){
    getIndex($b_ref,$s_ref,$I-1,$J,$index1_ref,$index2_ref,$lcs_ref);
  }
  else{
    getIndex($b_ref,$s_ref,$I,$J-1,$index1_ref,$index2_ref,$lcs_ref);
  }
}

