#!/usr/bin/perl

use Getopt::Std;

%option = ();
getopts("d:t:s:o:",\%option);

if( (!$option{d}) || (!$option{t}) || (!$option{s}) || (!$option{o}) ){
  print "Usage:\n";
  print "graphThresh.pl -d <data file> -t <threshold> -s <separate points file> -o <output file>\n\n";
  print "<data file> : contains the top right half of the adjacency matrix\n";
  print "              in 3 column format\n\n";
  print "<threshold> : the edge-weight threshold (between 0 and 1)\n";
  print "              all edges with lower weights than threshold are chopped off\n\n"; 
  print "<separate points file> : the vertex numbers of the isolated points are written.\n";
  print "<output file> : where the full adjacency matrix of the reduced graph, \n";
  print "                i.e., the one without isolated points, is written\n";   
  exit(1);
}

$edgeCount = 0;
$type = 1;
$maxVertex = 0;
$minVertex = 10;
%connectedVertex = ();

open(DATA,$option{d}) || die "couldn't open $option{d}\n";

while(<DATA>){
  chomp;
  ($vertex1,$vertex2,$weight)=split;
  if($minVertex > $vertex1){
    $minVertex = $vertex1;
  }
  if($maxVertex < $vertex2){
    $maxVertex = $vertex2;
  }
  unless($weight < $option{t}){
    push @{ $graph{$vertex1} }, $vertex2, $weight;
    push @{ $graph{$vertex2} }, $vertex1, $weight;
    $edgeCount++;
    
    $connectedVertex{$vertex1} = 1;
    $connectedVertex{$vertex2} = 1;
  }
}
close(DATA);



open(POINTS,">$option{s}") || die "couldn't open $option{s}\n";
%mapVertex = ();
$vertexCount=0; 

for $vertex ($minVertex..$maxVertex){
  if(defined($connectedVertex{$vertex})){
    $mapVertex{$vertex} = $vertexCount;
    $vertexCount++;
  }
  else{
    print POINTS "$vertex\n";
  }
}

close(POINTS);
print "The graph of ".($maxVertex-$minVertex+1)." vertices gave :\n";
print "           - a [smaller] graph of ".$vertexCount." vertices\n";  
print "           - ".($maxVertex-$minVertex+1-$vertexCount)." disconnected vertices\n";

open(OUT,">$option{o}") || die "couldn't open $option{o}\n";
foreach $vertex (sort { $a <=> $b} keys %mapVertex){ 
  $whichOne = 1;
  foreach $entry (0 .. $#{ $graph{$vertex} }) {
    if($whichOne == 1){
      print OUT "$mapVertex{$vertex} $mapVertex{$graph{$vertex}[$entry]} ";
      $whichOne = 2;
    }
    else{
      print OUT "$graph{$vertex}[$entry]\n";
      $whichOne = 1;
    }
  }
}
