#!/usr/bin/perl

use Getopt::Std;

%option = ();
getopts("fd:s:o:",\%option);

if( (!$option{d}) || (!$option{o}) ){
  print "Usage:\n";
  print "makeMetisGraph.pl [-f ] -d <data file> -o <output file>\n\n";
  print " -f  : an optional argument. Adjacency matrix is assumed to be full if -f is present\n";
  print "       if absent, the program assumes the top-right half of the adjacency matrix as input\n\n";
  print "<data file> : the adjacency matrix\n";
  print "              in 3 column format\n\n";
  exit(1);
}

$edgeCount = 0;
$type = 1;
$maxVertex = 0;
$minVertex = 10;
%connectedVertex = ();
$MaxInt = 2**32 -1;

open(DATA,$option{d}) || die "couldn't open $option{d}\n";

$weightSum = 0;
while(<DATA>){
  chomp;
  ($vertex1,$vertex2,$weight)=split;
  if($minVertex > $vertex1){
    $minVertex = $vertex1;
  }
  if($maxVertex < $vertex2){
    $maxVertex = $vertex2;
  }
  $semiFinalWeight = $weight;
  push @{ $graph{$vertex1} }, $vertex2, $semiFinalWeight;
  unless($option{f}){
    push @{ $graph{$vertex2} }, $vertex1, $semiFinalWeight;
  }
  $weightSum += $semiFinalWeight;
  $edgeCount++;
  $connectedVertex{$vertex1} = 1;
  $connectedVertex{$vertex2} = 1;
}
close(DATA);
if($option{f}){
  $edgeCount /= 2;
}

%mapVertex = ();
$vertexCount=1;

for $vertex ($minVertex..$maxVertex){
  if(defined($connectedVertex{$vertex})){
    $mapVertex{$vertex} = $vertexCount;
    $vertexCount++;
  }
}
$vertexCount--;
print "The input graph of ".($maxVertex-$minVertex+1)." vertices had ".($maxVertex-$minVertex+1-$vertexCount)." disconnected vertices\n";


$scaleUp = 0.1*$MaxInt/$weightSum;

open(OUT,">$option{o}") || die "couldn't open $option{o}\n";
print OUT "$vertexCount $edgeCount $type\n";
foreach $vertex (sort { $a <=> $b} keys %mapVertex){ 
  $whichOne = 1;
  foreach $entry (0 .. $#{ $graph{$vertex} }) {
    if($whichOne == 1){
      print OUT "$mapVertex{$graph{$vertex}[$entry]} ";
      $whichOne = 2;
    }
    else{
      $val = int $scaleUp*$graph{$vertex}[$entry];
      print OUT "$val ";
      $whichOne = 1;
    }
  }
  print OUT "\n";
}
