#!/usr/bin/perl

use Getopt::Std;

%option = ();
getopts("d:s:o:c:t:p:uh:",\%option);

if( (!$option{d}) || (!$option{s}) ){
  print "Usage:\n";
  print "wsct.pl  -d <data dir> -s <project name> [-o <output dir>] [-c <# of Cluster>]\n";
  print "        [-t <basic threshold>] [-p <similarity power>] [-u <unweighted>]\n";
  print "        [-h <theta threshold>]\n\n";
  print "-d <data dir>     : directory in which the data files will be present\n";
  print "-s <project name> : basic project/dataset name\n";
  print "-o <output dir>   : output directory. [default <data dir>]\n";
  print "-c <# of clusters>   : number of clusters [default 25] \n";  
  print "-t <basic threshold> : basic cutoff threshold for similarity graph [deafult 0.1]\n";
  print "-p <similarity power>: importance component exponent [default 1]\n";
  print "-u <unweighted>      : are the sequences unweighted [default no]\n";
  print "-h <theta threshold> : the outlier cutoff threshold for similarity graph [default 0.3]\n";
  exit(1);
}

$pathDir = $option{d};
$pathDir =~ s/\/$//g;
$system = $option{s};

if(defined($option{o})) {
  $outDir = $option{o};
  $outDir =~ s/\/$//g;
}
else{
  $outDir = $pathDir;
}


#--------------------------------------------
# Get Sparse Path Similarity Matrix

if(defined($option{t})){
  $thres = $option{t};
}
else{
  $thres = 0.1;
}

if(defined($option{p})){
  $power = $option{p};
}
else{
  $power = 1;
}

$IsWeighted = 1;
if(defined($option{u})){
  $IsWeighted = 0;
}

print "STEP 1 : Generating the Similarity Graph ...this may take some time\n";
$command = "LCS_sim.pl -d $pathDir/sequence.$system.data -t $thres -p $power -u $IsWeighted -o $outDir/tmp/sim.$system.$power";
$stat = 0xffff & system($command);
checkstatus($stat);


#--------------------------------------------
# Graph Thresholding Operation

if(defined($option{h})){
  $theta = $option{h};
}
else{
  $theta = 0.25;
}

print "STEP 2 : Thresholding the Similarity Graph ...theta = $theta \n";
$command = "graphThresh.pl -d $outDir/tmp/sim.$system.$power -t $theta -s $outDir/tmp/sep.$system -o $outDir/tmp/sim.$system.new";
$stat = 0xffff & system($command);
checkstatus($stat);

#--------------------------------------------
# Graph Partitioning

print "STEP 3(a) : Converting the Graph format ... \n";
$command = "makeMetisGraph.pl -f -d $outDir/tmp/sim.$system.new -o $outDir/tmp/sim.$system.graph";
$stat = 0xffff & system($command);
checkstatus($stat);


if(defined($option{c})){
  $numCluster = $option{c};
}
else{
  $numCluster = 25;
}

print "STEP 4(b) : Graph Partitioning ...$numCluster parts \n";
$command = "pmetis $outDir/tmp/sim.$system.graph $numCluster\n";
$stat = 0xffff & system($command);
#checkstatus($stat);

#----------------------------------------------
#  The Isolated Points

print "STEP 5(a) : separating out isolated paths ...\n";
$command = "splinput.pl -p $outDir/sequence.$system.data -s $outDir/tmp/sep.$system -u $outDir/tmp/unique.$system -o $outDir/tmp/set.$system";
$stat = 0xffff & system($command);
checkstatus($stat);

#----------------------------------------------
#  Make clusters readable : step 1

print "STEP 5(b) : formatting results : 1...";
$command = "getClusters.pl -d $outDir/tmp/sim.$system.graph.part.$numCluster -p $outDir/tmp/set.$system -c $pathDir/alphabet.$system.txt -o $outDir/tmp/int1.results.tmp -i $outDir/tmp/unique.$system -f $outDir/isolated.$system.results -u $IsWeighted";
$stat = 0xffff & system($command);
checkstatus($stat);


#-----------------------------------------------
#  Make Clusters readable : step 2

print "2 ...";
$command = "giveResults.pl -d $outDir/tmp/int1.results.tmp -o $outDir/clusters.$system.results";
$stat = 0xffff & system($command);
checkstatus($stat);
print "done !\n";

print "\nResults :\n";
print "Isolated sessions :  $outDir/isolated.$system.results\n";
print "Clustered sessions : $outDir/clusters.$system.results\n";


sub checkstatus{
  my $rc = shift;
  unless($rc == 0){
    print "command failed: $!\n";
    exit(1);
  }
}

